
# q-expansion of newform 3136.2.a.b, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3136
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-3], [1], [0], [1], [-2], [3], [-5], [-3], [6], [-1], [5], [-10], [4], [1], [9], [-3], [-3], [-11], [16], [7], [-11], [4], [-9], [6], [13], [5], [3], [-11], [-10], [-8], [-17], [3], [-4], [-15], [15], [-15], [-9], [-20], [21], [1], [-22], [17], [-5], [-18], [-9], [-12], [24], [-7], [-7], [-13], [-4], [-17], [-24], [-13], [3], [17], [-3], [-7], [-18], [17], [-6], [-4], [11], [31], [-27], [7], [14], [-3], [6], [-5], [-15], [19], [-19], [16], [9], [-19], [17], [3], [19], [-20], [-10], [-41], [-26], [-15], [27], [-2], [-17], [6], [-16], [-25], [-21], [-13], [0], [7], [16], [-7], [15], [-13], [25], [0], [-11], [-11], [-1], [17], [31], [-12], [43], [-21], [-34], [-7], [21], [22], [5], [0], [15], [44], [43], [5], [36], [1], [-2], [9], [-39], [47], [18], [-27], [29], [16], [-11], [41], [32], [-47], [-34], [27], [14], [-35], [13], [-42], [-25], [20], [25], [-21], [4], [37], [-24], [22], [-57], [-5], [37], [-39], [18], [-28], [-55], [13], [-48], [17], [-41], [-38], [-39], [-49], [30], [32], [-57], [-37], [-33], [59], [-31], [-58], [21], [-51], [14], [45], [-1], [-17], [54], [8], [-7], [0], [25], [37], [-60], [58], [39], [-13], [33], [21], [-31], [19], [-25], [-1], [44], [11], [33], [-15], [-21], [11], [33], [14], [-27], [29], [8], [-3], [-45], [-20], [-31], [21], [19], [-6], [45], [59], [30], [8], [63], [15], [-37], [39], [51], [9], [2], [1], [-20], [7], [-15], [20], [6], [-21], [-13], [73], [-25], [49], [-51], [-16], [19], [-28], [-37], [11], [73], [36], [-28], [-59], [-29], [-61], [61], [-30], [-51], [56], [41], [41], [-12], [62], [15], [7], [62], [9], [3], [17], [-47], [22], [27], [-29], [-36], [-43], [-26], [67], [-61], [-50], [-58], [36], [61], [-55], [9], [35], [-71], [54], [-43], [7], [-79], [31], [27], [-9], [79], [-56], [46], [3], [-55], [47], [-34], [83], [70], [45], [-63], [7], [0], [14], [5], [49], [6], [-41], [16], [15], [-7], [-25], [84], [43], [-50], [5], [38], [9], [1], [-75], [65], [35], [-67], [72], [-5], [68], [-9], [-2], [-10], [73], [67], [-58], [-20], [31], [55], [7], [55], [-3], [-74], [85], [-19], [40], [-11], [17], [-28], [-66], [51], [58], [-19], [-11], [-18], [54], [16], [29], [-76], [65], [-69], [68], [77], [-9], [3], [-10], [-21], [-83], [-38], [-11], [-39], [3], [-56], [-18], [-61], [31], [31], [5], [-21], [-18], [55], [-6], [-3], [-5], [49], [10], [-97], [-59], [-15], [0], [-69], [-57], [-2], [-51], [-42], [-72], [39], [60], [-9], [-77], [-63], [-1], [-32], [50], [-23], [45], [7], [25], [39], [11], [-26], [-76], [-11], [5], [99], [57], [7], [85], [-83], [57], [-18], [-55], [15], [-73], [-3], [-60], [-93], [-10], [-35], [-71], [103], [30], [57], [-15], [5], [48], [-100], [-42], [-71], [43], [30], [21], [63]]
